#
#  sundtek control center
#  coded by giro77
#  support: http://www.i-have-a-dreambox.com
#
#
#  This plugin is licensed under the Creative Commons 
#  Attribution-NonCommercial-ShareAlike 3.0 Unported License.
#  To view a copy of this license, please visit
#  http://creativecommons.org/licenses/by-nc-sa/3.0/ or send a letter to Creative
#  Commons, 559 Nathan Abbott Way, Stanford, California 94305, USA.
#
#  Alternatively, this plugin may be distributed 
#  with devices from sundtek ltd. or sundtek germany.
#
#  This plugin is NOT free software. It is open source, you are allowed to
#  modify it (if you keep the license), but it may not be commercially 
#  distributed other than under the conditions noted above. 
#
#  the software is provided "as is", without warranty of any kind,
#  express or implied, including but not limited to the warranties
#  of merchantability, fitness for a particular purpose and
#  noninfringement. in no event shall the authors or copyright
#  holders be liable for any claim, damages or other liability,
#  whether in an action of contract, tort or otherwise, arising
#  from, out of or in connection with the software or the use or
#  other dealings in the software.
#

from Components.ActionMap import ActionMap
from Components.config import config, getConfigListEntry, ConfigSubsection, ConfigInteger, ConfigYesNo, ConfigText, ConfigSelection, configfile
from Components.ConfigList import ConfigListScreen
from Components.Input import Input
from Components.Label import Label
from Components.MenuList import MenuList
from Components.MultiContent import MultiContentEntryText
from Components.NimManager import nimmanager
from Components.Sources.StaticText import StaticText

from enigma import eConsoleAppContainer, getEnigmaVersionString

from Plugins.Plugin import PluginDescriptor

from Screens.ChoiceBox import ChoiceBox
from Screens.Console import Console
from Screens.InputBox import InputBox
from Screens.Screen import Screen
from Screens.Standby import TryQuitMainloop
from Screens.MessageBox import MessageBox

from Tools.NumericalTextInput import NumericalTextInput
import os, re, time, datetime
import urllib 

# for localized texts
from . import _


## configs ################################################################

config.plugins.SundtekControlCenter = ConfigSubsection()
config.plugins.SundtekControlCenter.autostart = ConfigYesNo(default=False)

config.plugins.SundtekControlCenter.sunconf = ConfigSubsection()
config.plugins.SundtekControlCenter.sunconf.support = ConfigYesNo(default=False)
config.plugins.SundtekControlCenter.sunconf.vtuneracceleration = ConfigYesNo(default=False)
config.plugins.SundtekControlCenter.sunconf.autoupdate = ConfigYesNo(default=False)
config.plugins.SundtekControlCenter.sunconf.loglevel = ConfigSelection(default="0", choices = [("0", _("off")),("1", ("min")),("2", ("max"))])

config.plugins.SundtekControlCenter.dvbtransmission = ConfigSelection(default="0", choices = [("0", ("DVB-S/SVB-S2")),("1", ("DVB-C")),("2", ("DVB-T"))])
config.plugins.SundtekControlCenter.secondstick = ConfigYesNo(default=False)
config.plugins.SundtekControlCenter.secdvbtransmission = ConfigSelection(default="0", choices = [("0", ("DVB-S/SVB-S2")),("1", ("DVB-C")),("2", ("DVB-T"))])

config.plugins.SundtekControlCenter.usbnet = ConfigSubsection()
config.plugins.SundtekControlCenter.usbnet.selection = ConfigSelection(default="0", choices = [("0", _("via USB")),("1", _("via Network"))])
config.plugins.SundtekControlCenter.usbnet.networkip = ConfigText(default="0.0.0.0", visible_width = 50, fixed_size = False)

config.plugins.SundtekControlCenter.secusbnet = ConfigSubsection()
config.plugins.SundtekControlCenter.secusbnet.selection = ConfigSelection(default="0", choices = [("0", _("via USB")),("1", _("via Network"))])
config.plugins.SundtekControlCenter.secusbnet.networkip = ConfigText(default="0.0.0.0", visible_width = 50, fixed_size = False)

## version string #########################################################

sundtekcontrolcenter_version = "1.3.5-20121210-r1"
global sundtekcontrolcenter_version

###########################################################################

class SundtekControlCenter(Screen, ConfigListScreen):
   
    s = (r"(Tuner .:) (.*Sundtek.*) \((DVB-.*)\)")
    pattern = re.compile(s)
    nims = nimmanager.nimList() # get nim_sockets
    result = []
    i=0
    for item in nims:
        if str(re.findall(s, str(nims[i]))) != "[]":
             result.append(re.findall(s, str(nims[i])))
        i+=1
    
    imageone =""
    imagetwo =""
    
    if len(result) == 1:
        if (result[0][0][2]=="DVB-C") or (result[0][0][2]=="DVB-T"):
            imageone="/usr/lib/enigma2/python/Plugins/Extensions/SundtekControlCenter/sundtek_dvbc.png"
        else:
            imageone="/usr/lib/enigma2/python/Plugins/Extensions/SundtekControlCenter/sundtek_dvbs.png"
            
        skin = "<screen title=\"SundtekControlCenter\" position=\"100,100\" size=\"750,550\" name=\"SundtekControlCenter\">\
                <ePixmap pixmap=\"/usr/lib/enigma2/python/Plugins/Extensions/SundtekControlCenter/bg.png\" position=\"0,0\" size=\"750,124\" alphatest=\"on\"/>\
                <ePixmap pixmap=\"skin_default/buttons/red.png\" position=\"75,385\" size=\"140,40\" alphatest=\"on\" />\
                <ePixmap pixmap=\"skin_default/buttons/green.png\" position=\"225,385\" size=\"140,40\" alphatest=\"on\" />\
                <ePixmap pixmap=\"skin_default/buttons/yellow.png\" position=\"375,385\" size=\"140,40\" alphatest=\"on\" />\
                <ePixmap pixmap=\"skin_default/buttons/blue.png\" position=\"525,385\" size=\"140,40\" alphatest=\"on\" />\
                <widget name=\"btt_red\" position=\"75,385\" zPosition=\"1\" size=\"140,40\" font=\"Regular;20\" halign=\"center\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" />\
                <widget name=\"btt_green\" position=\"225,385\" zPosition=\"1\" size=\"140,40\" font=\"Regular;20\" halign=\"center\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" />\
                <widget name=\"btt_yellow\" position=\"375,385\" zPosition=\"1\" size=\"140,40\" font=\"Regular;20\" halign=\"center\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" />\
                <widget name=\"btt_blue\" position=\"525,385\" zPosition=\"1\" size=\"140,40\" font=\"Regular;20\" halign=\"center\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" /> \
                <widget name=\"ok\" position=\"25,430\" zPosition=\"1\" size=\"630,40\" font=\"Regular;16\" halign=\"left\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" />\
                <widget name=\"menu\" position=\"25,450\" zPosition=\"1\" size=\"630,40\" font=\"Regular;16\" halign=\"left\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" />\
                <widget name=\"infos\" position=\"25,470\" zPosition=\"1\" size=\"630,40\" font=\"Regular;16\" halign=\"left\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" />\
                <widget name=\"bouquets\" position=\"25,490\" zPosition=\"1\" size=\"630,40\" font=\"Regular;16\" halign=\"left\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" />\
                <widget name=\"netservers\" position=\"25,510\" zPosition=\"1\" size=\"630,40\" font=\"Regular;16\" halign=\"left\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" />\
                <widget name=\"driverupdateavail\" position=\"375,37\" zPosition=\"1\" size=\"350,30\" font=\"Regular;16\" halign=\"right\" valign=\"center\" transparent=\"1\" shadowColor=\"black\" shadowOffset=\"-3,-3\" foregroundColor=\"#00ffffff\" />\
                <widget name=\"updateavail\" position=\"375,55\" zPosition=\"1\" size=\"350,30\" font=\"Regular;16\" halign=\"right\" valign=\"center\" transparent=\"1\" shadowColor=\"black\" shadowOffset=\"-3,-3\" foregroundColor=\"#00ffffff\" />\
                <widget name=\"version\" position=\"575,75\" zPosition=\"1\" size=\"150,30\" font=\"Regular;16\" halign=\"right\" valign=\"center\" transparent=\"1\" shadowColor=\"black\" shadowOffset=\"-3,-3\" />\
                <widget name=\"config\" position=\"260,135\" size=\"460,245\" scrollbarMode=\"showOnDemand\" zPosition=\"1\"/>\
                <ePixmap position=\"35,130\" size=\"96,58\" pixmap=\""+ imageone +"\" transparent=\"1\" alphatest=\"on\" />\
                <widget name=\"tunerone\" position=\"25,190\" zPosition=\"1\" size=\"215,60\" font=\"Regular;16\" halign=\"left\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" />\
                <ePixmap position=\"625, 500\" size=\"100,40\" pixmap=\"/usr/lib/enigma2/python/Plugins/Extensions/SundtekControlCenter/plugin.png\" transparent=\"1\" alphatest=\"on\" />\
            </screen>"
    elif len(result) == 2:
        if (result[0][0][2]=="DVB-C") or (result[0][0][2]=="DVB-T"):
            imageone="/usr/lib/enigma2/python/Plugins/Extensions/SundtekControlCenter/sundtek_dvbc.png"
        else:
            imageone="/usr/lib/enigma2/python/Plugins/Extensions/SundtekControlCenter/sundtek_dvbs.png"
            
        if (result[1][0][2]=="DVB-C") or (result[1][0][2]=="DVB-T"):
            imagetwo="/usr/lib/enigma2/python/Plugins/Extensions/SundtekControlCenter/sundtek_dvbc.png"
        else:
            imagetwo="/usr/lib/enigma2/python/Plugins/Extensions/SundtekControlCenter/sundtek_dvbs.png"
            
        skin = "<screen title=\"SundtekControlCenter\" position=\"100,100\" size=\"750,550\" name=\"SundtekControlCenter\">\
                <ePixmap pixmap=\"/usr/lib/enigma2/python/Plugins/Extensions/SundtekControlCenter/bg.png\" position=\"0,0\" size=\"750,124\" alphatest=\"on\"/>\
                <ePixmap pixmap=\"skin_default/buttons/red.png\" position=\"75,385\" size=\"140,40\" alphatest=\"on\" />\
                <ePixmap pixmap=\"skin_default/buttons/green.png\" position=\"225,385\" size=\"140,40\" alphatest=\"on\" />\
                <ePixmap pixmap=\"skin_default/buttons/yellow.png\" position=\"375,385\" size=\"140,40\" alphatest=\"on\" />\
                <ePixmap pixmap=\"skin_default/buttons/blue.png\" position=\"525,385\" size=\"140,40\" alphatest=\"on\" />\
                <widget name=\"btt_red\" position=\"75,385\" zPosition=\"1\" size=\"140,40\" font=\"Regular;20\" halign=\"center\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" />\
                <widget name=\"btt_green\" position=\"225,385\" zPosition=\"1\" size=\"140,40\" font=\"Regular;20\" halign=\"center\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" />\
                <widget name=\"btt_yellow\" position=\"375,385\" zPosition=\"1\" size=\"140,40\" font=\"Regular;20\" halign=\"center\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" />\
                <widget name=\"btt_blue\" position=\"525,385\" zPosition=\"1\" size=\"140,40\" font=\"Regular;20\" halign=\"center\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" /> \
                <widget name=\"ok\" position=\"25,430\" zPosition=\"1\" size=\"630,40\" font=\"Regular;16\" halign=\"left\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" />\
                <widget name=\"menu\" position=\"25,450\" zPosition=\"1\" size=\"630,40\" font=\"Regular;16\" halign=\"left\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" />\
                <widget name=\"infos\" position=\"25,470\" zPosition=\"1\" size=\"630,40\" font=\"Regular;16\" halign=\"left\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" />\
                <widget name=\"bouquets\" position=\"25,490\" zPosition=\"1\" size=\"630,40\" font=\"Regular;16\" halign=\"left\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" />\
                <widget name=\"netservers\" position=\"25,510\" zPosition=\"1\" size=\"630,40\" font=\"Regular;16\" halign=\"left\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" />\
                <widget name=\"driverupdateavail\" position=\"375,37\" zPosition=\"1\" size=\"350,30\" font=\"Regular;16\" halign=\"right\" valign=\"center\" transparent=\"1\" shadowColor=\"black\" shadowOffset=\"-3,-3\" foregroundColor=\"#00ffffff\" />\
                <widget name=\"updateavail\" position=\"375,55\" zPosition=\"1\" size=\"350,30\" font=\"Regular;16\" halign=\"right\" valign=\"center\" transparent=\"1\" shadowColor=\"black\" shadowOffset=\"-3,-3\" foregroundColor=\"#00ffffff\" />\
                <widget name=\"version\" position=\"575,75\" zPosition=\"1\" size=\"150,30\" font=\"Regular;16\" halign=\"right\" valign=\"center\" transparent=\"1\" shadowColor=\"black\" shadowOffset=\"-3,-3\" />\
                <widget name=\"config\" position=\"260,135\" size=\"460,245\" scrollbarMode=\"showOnDemand\" zPosition=\"1\"/>\
                <ePixmap position=\"35,130\" size=\"96,58\" pixmap=\""+ imageone +"\" transparent=\"1\" alphatest=\"on\" />\
                <widget name=\"tunerone\" position=\"25,190\" zPosition=\"1\" size=\"215,60\" font=\"Regular;16\" halign=\"left\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" />\
                <ePixmap position=\"35,255\" size=\"96,58\" pixmap=\""+ imagetwo +"\" transparent=\"1\" alphatest=\"on\" />\
                <widget name=\"tunertwo\" position=\"25,315\" zPosition=\"1\" size=\"215,60\" font=\"Regular;16\" halign=\"left\" valign=\"center\" shadowColor=\"black\" shadowOffset=\"-1,-1\" transparent=\"1\" />\
                <ePixmap position=\"625, 500\" size=\"100,40\" pixmap=\"/usr/lib/enigma2/python/Plugins/Extensions/SundtekControlCenter/plugin.png\" transparent=\"1\" alphatest=\"on\" />\
            </screen>"
    else:
        skin = """
            <screen title="SundtekControlCenter" position="100,100" size="750,550" name="SundtekControlCenter">
                <ePixmap pixmap="/usr/lib/enigma2/python/Plugins/Extensions/SundtekControlCenter/bg.png" position="0,0" size="750,124" alphatest="on"/>
                <ePixmap pixmap="skin_default/buttons/red.png" position="75,385" size="140,40" alphatest="on" />
                <ePixmap pixmap="skin_default/buttons/green.png" position="225,385" size="140,40" alphatest="on" />
                <ePixmap pixmap="skin_default/buttons/yellow.png" position="375,385" size="140,40" alphatest="on" />
                <ePixmap pixmap="skin_default/buttons/blue.png" position="525,385" size="140,40" alphatest="on" />
                <widget name="btt_red" position="75,385" zPosition="1" size="140,40" font="Regular;20" halign="center" valign="center" shadowColor="black" shadowOffset="-1,-1" transparent="1" />
                <widget name="btt_green" position="225,385" zPosition="1" size="140,40" font="Regular;20" halign="center" valign="center" shadowColor="black" shadowOffset="-1,-1" transparent="1" />
                <widget name="btt_yellow" position="375,385" zPosition="1" size="140,40" font="Regular;20" halign="center" valign="center" shadowColor="black" shadowOffset="-1,-1" transparent="1" />
                <widget name="btt_blue" position="525,385" zPosition="1" size="140,40" font="Regular;20" halign="center" valign="center" shadowColor="black" shadowOffset="-1,-1" transparent="1" /> 
                <widget name="ok" position="25,430" zPosition="1" size="630,40" font="Regular;16" halign="left" valign="center" shadowColor="black" shadowOffset="-1,-1" transparent="1" />
                <widget name="menu" position="25,450" zPosition="1" size="630,40" font="Regular;16" halign="left" valign="center" shadowColor="black" shadowOffset="-1,-1" transparent="1" />
                <widget name="infos" position="25,470" zPosition="1" size="630,40" font="Regular;16" halign="left" valign="center" shadowColor="black" shadowOffset="-1,-1" transparent="1" />
                <widget name="bouquets" position="25,490" zPosition="1" size="630,40" font="Regular;16" halign="left" valign="center" shadowColor="black" shadowOffset="-1,-1" transparent="1" />
                <widget name="netservers" position="25,510" zPosition="1" size="630,40" font="Regular;16" halign="left" valign="center" shadowColor="black" shadowOffset="-1,-1" transparent="1" />
                <widget name="driverupdateavail" position="375,37" zPosition="1" size="350,30" font="Regular;16" halign="right" valign="center" transparent="1" shadowColor="black" shadowOffset="-3,-3" foregroundColor="#00ffffff" />
                <widget name="updateavail" position="375,55" zPosition="1" size="350,30" font="Regular;16" halign="right" valign="center" transparent="1" shadowColor="black" shadowOffset="-3,-3" foregroundColor="#00ffffff" />
                <widget name="version" position="575,75" zPosition="1" size="150,30" font="Regular;16" halign="right" valign="center" transparent="1" shadowColor="black" shadowOffset="-3,-3" />
                <widget name="config" position="260,135" size="460,245" scrollbarMode="showOnDemand" zPosition="1"/>
                <widget name="tunerone" position="25,190" zPosition="1" size="215,60" font="Regular;16" halign="center" valign="center" shadowColor="black" shadowOffset="-1,-1" transparent="1" />
                <ePixmap position="625, 500" size="100,40" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/SundtekControlCenter/plugin.png" transparent="1" alphatest="on" />
            </screen>"""
            
                
    def __init__(self, session, args=0):
        Screen.__init__(self, session)
        ConfigListScreen.__init__(self, [])
        self.updateSettingList()
        
        
        ### get nim_socket informations 
        s = (r"(Tuner .:) (.*Sundtek.*) \((DVB-.*)\)")
        pattern = re.compile(s)

        nims = nimmanager.nimList() # get nim_sockets
        result = []
        i=0
        for item in nims:
            if str(re.findall(s, str(nims[i]))) != "[]":
                result.append(re.findall(s, str(nims[i])))
            i+=1

        ### search for sundtekcontrolcenter updates   
        try: 
            version = urllib.urlopen('http://shockstyle.de/sundtek/version.html').read()
        except:
            version = "n/a"

        if version == sundtekcontrolcenter_version:
            self["updateavail"] = Label("")
        elif version == "n/a":
            self["updateavail"] = Label("n/a")
        else:
            self["updateavail"] = Label(_("scc update available"))
            

        ### search for sundtek driver update

        s = r"(?P<year>\d{2})(?P<month>\d{2})(?P<day>\d{2}).(?P<hours>\d{2})(?P<minutes>\d{2})(?P<seconds>\d{2})"
        pattern = re.compile(s)
        try:
            netdriver = urllib.urlopen('http://sundtek.de/media/latest.phtml').read()
        except:
            netdriver = "n/a"

        match = pattern.search(netdriver)
        if match:
            netdriver = match.group("year")+match.group("month")+match.group("day")+match.group("hours")+match.group("minutes")+match.group("seconds")
        else:
            netdriver = "n/a"

        s = r"(?P<year>\d{2})-(?P<month>\d{2})-(?P<day>\d{2}) (?P<hours>\d{2}):(?P<minutes>\d{2}):(?P<seconds>\d{2})"
        pattern = re.compile(s)
        if os.path.exists("/opt/bin/mediaclient"):
            installeddriver = os.popen("/opt/bin/mediaclient --build", "r").read()
        else:
            installeddriver ="n/a"

        match = pattern.search(installeddriver)
        if match:
            installeddriver = match.group("year")+match.group("month")+match.group("day")+match.group("hours")+match.group("minutes")+match.group("seconds")
        else:
            installeddriver = "n/a"
        
        if (netdriver != "n/a") and (installeddriver != "n/a"):
            if int(netdriver) <= int(installeddriver):
                self["driverupdateavail"] = Label("")
            else:
                self["driverupdateavail"] = Label(_("driver update available"))
        else:
            self["driverupdateavail"] = Label("n/a")


        ###
        self["btt_red"] = Label(_("Exit"))
        self["btt_green"] = Label(_("Setup"))
        self["btt_yellow"] = Label(_("Restart"))
        self["btt_blue"] = Label(_("Start Tuner"))
        self["ok"] = Label(_("OK/ green = activate settings"))
        self["menu"] = Label(_("Menu = call further functions..."))
        self["infos"] = Label(_("Info = show tuner informations"))
        self["bouquets"] = Label(_("Bouquet + = install or update driver"))
        self["netservers"] = Label(_("Bouquet - = scan for IPTV server addresses"))
        self["version"] = Label(sundtekcontrolcenter_version)

        self["tunerone"] = Label("")
        self["tunertwo"] = Label("")
        
        ### get tunerinformations
        if len(result) == 1:
            self["tunerone"] = Label(str(result[0][0][0]+" "+str(result[0][0][2])+"\n"+str(result[0][0][1])))
        elif len(result) == 2:
            self["tunerone"] = Label(str(result[0][0][0]+" "+str(result[0][0][2])+"\n"+str(result[0][0][1])))
            self["tunertwo"] = Label(str(result[1][0][0]+" "+str(result[1][0][2])+"\n"+str(result[1][0][1])))
        else:
            self["tunerone"] = Label(_("No stick found"))
        ###
               
        self["actions"] = ActionMap(["MenuActions", "OkCancelActions", "ChannelSelectBaseActions", "ColorActions","ChannelSelectEPGActions"], 
        {
            "menu": self.menu,
            "ok": self.save,
            "cancel": self.cancel,
            "red": self.cancel,
            "green": self.save,
            "yellow": self.howtorestart,
            "blue": self.tunerstart,
            "showEPGList": self.dvbinfo,
            "nextBouquet": self.checkdriverversion,
            "prevBouquet": self.scannetwork,
        },-2)
        
        self.onLayoutFinish.append(self.layoutFinished)

    ########################################################       
       
       
    def keyLeft(self):
        ConfigListScreen.keyLeft(self)
        self.updateSettingList()

    def keyRight(self):
        ConfigListScreen.keyRight(self)
        self.updateSettingList()
        
    def save(self):
        for x in self["config"].list:
            x[1].save()
            
        configfile.save()
        self.sundtekconfigfile()
        self.setsettings()
        
    def cancel(self):
        for x in self["config"].list:
            x[1].cancel()
        self.close(False, self.session)
        
    def menu(self):
        options = [
            (_("Install or Update sundtek Driver"), self.checkdriverversion),
            (_("Scan for IPTV server addresses"), self.scannetwork),
            (_("Service Mode"), self.enableservicemode),
            (_("Show tuner informations"), self.dvbinfo),
            (_("Backup Drivers to HDD"), self.driverbackup),
            (_("Restore Drivers from HDD"), self.driverrestore),
            (_("Check for a newer version of the SundtekControlCenter"), self.selfupdate),
            (_("Help"), self.helper),            
        ]
        
        self.session.openWithCallback(self.thismenuCallback, ChoiceBox, list = options)
        
    def thismenuCallback(self,ret):
        ret and ret[1]()
    
    def updateSettingList(self):
        list = [] ### creating list

        list.append(getConfigListEntry(_("Autostart"), config.plugins.SundtekControlCenter.autostart))
        list.append(getConfigListEntry(_("DVB Transmission Way"), config.plugins.SundtekControlCenter.dvbtransmission)) 
        list.append(getConfigListEntry(_("* USB/Network"), config.plugins.SundtekControlCenter.usbnet.selection)) 
        
        if config.plugins.SundtekControlCenter.usbnet.selection.value == "1": ## if networking then add ip mask to list
            sublist = [
                getConfigListEntry(_("      Network IP"), config.plugins.SundtekControlCenter.usbnet.networkip)
            ]
            
            list.extend(sublist)

        
        list.append(getConfigListEntry(_("* Configuration support"), config.plugins.SundtekControlCenter.sunconf.support))
        
        if str(config.plugins.SundtekControlCenter.sunconf.support.value) == "True":
            sublist = [
                getConfigListEntry(_("      Loglevel"), config.plugins.SundtekControlCenter.sunconf.loglevel),
                getConfigListEntry(_("      Driver autoupdate"), config.plugins.SundtekControlCenter.sunconf.autoupdate),
                getConfigListEntry(_("      vTuner acceleration for DM800"), config.plugins.SundtekControlCenter.sunconf.vtuneracceleration),
            ]
            
            list.extend(sublist)
                

        list.append(getConfigListEntry(_("* Use Second Stick"), config.plugins.SundtekControlCenter.secondstick))
        
        if str(config.plugins.SundtekControlCenter.secondstick.value) == "True": 
            list.append(getConfigListEntry(_("      DVB Transmission Way, Stick 2"), config.plugins.SundtekControlCenter.secdvbtransmission)) 
            list.append(getConfigListEntry(_("*     USB/Network"), config.plugins.SundtekControlCenter.secusbnet.selection))
            
            if config.plugins.SundtekControlCenter.secusbnet.selection.value == "1": ## if networking then add ip mask to list
                secsublist = [
                    getConfigListEntry(_("          Network IP"), config.plugins.SundtekControlCenter.secusbnet.networkip)
                ]
                list.extend(secsublist)
        else: 
            config.plugins.SundtekControlCenter.secusbnet.selection.value = "0"            

        self["config"].list = list
        self["config"].l.setList(list)
    
    def layoutFinished(self):
        self.setTitle(_("Sundtek Control Center"))
              
    def selfupdate(self):
        try: 
            self.version = urllib.urlopen('http://shockstyle.de/sundtek/version.html').read()
        except:
            self.version = "n/a"

        if self.version == sundtekcontrolcenter_version:
            self.session.open(MessageBox, _("Latest version is already installed"), MessageBox.TYPE_INFO, 7)
        elif self.version == "n/a":
            self.session.open(MessageBox, _("Check not possible. Please ensure that the Box is connected to the internet."), MessageBox.TYPE_INFO, 7)
        else:
            self.session.openWithCallback(self.updateplugin, MessageBox, _("A newer version of the sundtekcontrolcenter plugin is available. Install now?"), MessageBox.TYPE_YESNO)         
            
    def updateplugin(self, result):
        if result:
            self.session.open(MessageBox, _("The SundtekControlCenter is updated in background"), MessageBox.TYPE_INFO, 7)
            self.container=eConsoleAppContainer()
            self.container.appClosed.append(self.finishedSelfUpdate)
            self.container.execute("opkg install --force-overwrite " + "http://shockstyle.de/sundtek/enigma2-plugin-extensions-sundtekcontrolcenter_"+str(self.version)+"_all.ipk")

        
    def finishedSelfUpdate(self, result):
        self.session.openWithCallback(self.restartbox, MessageBox, _("SundtekControlCenter successfully updated! You need to restart Enigma2. Restart now?"), MessageBox.TYPE_YESNO) 
    
    def restartbox(self, result):
        if result:
			self.session.open(TryQuitMainloop, 3)
        
    def checkdriverversion(self):
        s = r"(?P<year>\d{2})(?P<month>\d{2})(?P<day>\d{2}).(?P<hours>\d{2})(?P<minutes>\d{2})(?P<seconds>\d{2})"
        pattern = re.compile(s)
        try:
            netdriver = urllib.urlopen('http://sundtek.de/media/latest.phtml').read() 
        except:
            netdriver = "Build date: n/a" 

        match = pattern.search(netdriver)
        if match:
            match = match.group("year")+" "+match.group("month")+" "+match.group("day")+" "+match.group("hours")+" "+match.group("minutes")+" "+match.group("seconds")
            sundtekdriverdate = time.strptime(match, "%y %m %d %H %M %S")
            netdriver = "Build date: "+time.strftime("%Y-%m-%d %H:%M:%S", sundtekdriverdate)
        else:
            netdriver = "Build date: n/a"

        if os.path.exists("/opt/bin/mediaclient"):
            installeddriver = os.popen("/opt/bin/mediaclient --build", "r").read()
        else:
            installeddriver ="Build date: n/a"
            
        self.session.openWithCallback(self.disclaimer, MessageBox, _("latest sundtek driver version:\n")+netdriver+"\n"+_("\nyour driver version:\n")+installeddriver+_("\nUpdate to current sundtek driver version?"), MessageBox.TYPE_YESNO)

    def disclaimer(self, result): 
        if result:
            self.prompt("/usr/lib/enigma2/python/Plugins/Extensions/SundtekControlCenter/sundtekinstall.sh")

    def parseforip(self):
        s = r"(%s)" % ("\.".join(['(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)']*4))
                
        pattern = re.compile(s)

        s = r"\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)    \| ([0-9])"
        pattern2 = re.compile(s)
        
        s = os.popen("/opt/bin/mediaclient --scan-network").read()
        
        i=0
        endip = []
        results = []

        while True:
            match = pattern.search(s,i)
               
            if match:
                endip.append(match.group(5))
                results.append(match.group())
                i = match.end()+1
            else:
                break
        
        i=0
        endip2 = []
        results2 = []

        while True:
            match = pattern2.search(s,i)
               
            if match:
                endip2.append(match.group(1))
                results2.append(match.group(2).rstrip())
                i = match.end()+1
            else:
                break              
                                          
        if len(endip2) == 1:
            if endip[0] == endip2[0]:
                results[0] = results[0]+":"+results2[0]
            else:
                results[1] = results[1]+":"+results2[0]
        
        elif len(results) >= 2 and len(results2) >= 1:
            results[0] = results[0]+":"+results2[0]            
            if len(results2) >= 2:
                results[1] = results[1]+":"+results2[1] 
        
        return results

    def scannetwork(self):
        if os.path.exists("/opt/bin/mediaclient"):
            if len(self.parseforip()) == 1:
                self.session.openWithCallback(self.usenetip, MessageBox, _("Use following address as media server?\n")+self.parseforip()[0], MessageBox.TYPE_YESNO)
            elif len(self.parseforip()) >= 2:
                self.session.openWithCallback(self.usetwonetip, MessageBox, _("Use following addresses?\n")+self.parseforip()[0]+"\n"+self.parseforip()[1], MessageBox.TYPE_YESNO)
            else:
                self.session.open(MessageBox, _("No IPTV media server found"), MessageBox.TYPE_INFO, 7)

    def usenetip(self, result): 
        if result:
            config.plugins.SundtekControlCenter.usbnet.selection.value = "1"
            config.plugins.SundtekControlCenter.usbnet.networkip.value = self.parseforip()[0]
            self.updateSettingList()
            
    def usetwonetip(self, result): 
        if result:
            config.plugins.SundtekControlCenter.usbnet.selection.value = "1"
            config.plugins.SundtekControlCenter.usbnet.networkip.value = self.parseforip()[0]
            
            config.plugins.SundtekControlCenter.secondstick.value = "True"
            config.plugins.SundtekControlCenter.secusbnet.selection.value = "1"
            config.plugins.SundtekControlCenter.secusbnet.networkip.value = self.parseforip()[1]
            self.updateSettingList()
            
                   
    def enableservicemode(self):
        self.session.openWithCallback(self.startservicemode, MessageBox, _("Service Mode starts a tunnel to the sundtek server. Please start service mode only if requested by sundtek support.\n\nStart Service Mode now?"), MessageBox.TYPE_YESNO)


    def startservicemode(self, result): 
        if result:
            os.popen("/opt/bin/mediaclient --portforward 23")
            self.prompt("/opt/bin/mediaclient --portforward 23")
                                                  
    def driverbackup(self):
        backuppath = "/media/hdd/backup/SundtekBackup"
        now = datetime.datetime.now()
        backupfile = "sundtek-"+now.strftime("%Y%m%d-%H%M")+".tar"

        if (not os.path.exists(backuppath)): # backup folder does not yet exist
            os.makedirs(backuppath)
        
        self.prompt("tar -czvf"+" "+backuppath+"/"+backupfile+" /usr/sundtek /opt/bin /etc/sundtek.conf /etc/sundtek.net") 
        
    def driverrestore(self):
        restorepath = "/media/hdd/backup/SundtekBackup"
        if (os.path.exists(restorepath)): # backup folder exists
            backupfile = [line.rstrip('\n') for line in (os.popen("ls "+restorepath+"/sundtek*tar").readlines())]
            if len(backupfile) > 0: # found at least one backup in the backupfolder
            
                options = []
                restorelist = []
                restorelist = [item.lstrip(restorepath) for item in backupfile]

                for item in backupfile:
                    options.append((_(restorelist.pop()), None))


                self.session.openWithCallback(self.restoremenuCallback, ChoiceBox, list = options)                    
                #self.session.open(MessageBox, str(options), MessageBox.TYPE_INFO)
                
            else: # es wurde kein backup file gefunden
                self.session.open(MessageBox, _("No backup file was found."), MessageBox.TYPE_INFO, 10)
        else: 
            self.session.open(MessageBox, _("No backups found."), MessageBox.TYPE_INFO, 10)
            
            
    def restoremenuCallback(self,ret):
        if ret:
            restore = ("tar -xvpzf /media/hdd/backup/SundtekBackup/%s -C /") % str(ret[0])
            self.prompt(restore) # restore from root
            self.session.open(MessageBox, _("Restoreing:  %s") % str(ret[0]), MessageBox.TYPE_INFO, 7)
        else:
            self.session.open(MessageBox, _("Restore was canceled"), MessageBox.TYPE_INFO, 7)
        
    def helper(self):
        self.prompt("cat /usr/lib/enigma2/python/Plugins/Extensions/SundtekControlCenter/helpfile.txt")

####################################################################

    def sundtekconfigfile(self):
    
        conffile ="/etc/sundtek.conf"
        now = datetime.datetime.now()
        s = r"ID: (\w+)"

        pattern = re.compile(s)
        #devices = os.popen("cat /usr/sundtek/test.txt").read()
        devices = os.popen("/opt/bin/mediaclient -e").read()
        
        i = 0
        results = []

        while True:
            match = pattern.search(devices,i)
            if match:
                results.append(match.group(1))
                i = match.end()+1
            else:
                break
    
        # header
        lineh1 = ("# sundtek configuration file - /etc/sundtek.conf\n")
        lineh2 = ("# created / modified: "+(now.strftime("%b %d %Y, %H:%Mh"))+" by sundtekcontrolcenter\n\n")
        header = lineh1+lineh2
        
        loglevel=""
        autoupdate=""
        vtuneracceleration=""
        
        if str(config.plugins.SundtekControlCenter.sunconf.support.value) == "True":
            loglevel = str(config.plugins.SundtekControlCenter.sunconf.loglevel.value) # loglevel
            if (loglevel == "1"):
                os.system("/opt/bin/mediaclient --loglevel=min")
                loglevel = "loglevel = min\n"
            elif (loglevel == "2"):
                os.system("/opt/bin/mediaclient --loglevel=max")
                loglevel = "loglevel = max\n"
            else:
                os.system("/opt/bin/mediaclient --loglevel=off")
                loglevel = "loglevel = off\n"
            
            autoupdate = str(config.plugins.SundtekControlCenter.sunconf.autoupdate.value) # driver autoupdate
            if (autoupdate == "True"):
                autoupdate = "autoupdate = on\n"
            else:
                autoupdate = "autoupdate = off\n"
            
            vtuneracceleration = str(config.plugins.SundtekControlCenter.sunconf.vtuneracceleration.value) # vtuner acceleration
            if (vtuneracceleration == "True"):
                vtuneracceleration = "vtuner_acceleration = on\n"
            else:
                vtuneracceleration = "vtuner_acceleration = off\n"
        

        networkingip = ""
        secnetworkingip = ""
        
        netsection = ""
        net1l1 = ""
        net1l2 = ""
        

        if (config.plugins.SundtekControlCenter.usbnet.selection.value == "1"):
            networkingip=config.plugins.SundtekControlCenter.usbnet.networkip.value
            networkingip.lstrip().rstrip()
            net1l1 = "###### configuration network\n[NETWORK]\ndevice="+networkingip+"\n"
        
        if str(config.plugins.SundtekControlCenter.secondstick.value) == "True":
            if (config.plugins.SundtekControlCenter.secusbnet.selection.value == "1"):
                secnetworkingip=config.plugins.SundtekControlCenter.secusbnet.networkip.value
                secnetworkingip.lstrip().rstrip()
                if (config.plugins.SundtekControlCenter.usbnet.selection.value == "1"):
                    net1l2 = "device="+secnetworkingip+"\n"
                else:
                    net1l2 = "###### configuration network\n[NETWORK]\ndevice="+secnetworkingip+"\n"
        
        netsection = net1l1+net1l2+"\n"
        
        stick1 = ""
        stick2 = ""
        twotuners = False
        
        #stick 1 data
        if len(results) >= 1:
            lines1l1 = ("###### configuration stick 1\n") 
            if (not config.plugins.SundtekControlCenter.usbnet.selection.value == "1"):
                lines1l2 = ("["+results.pop(0)+"]\n")
            elif config.plugins.SundtekControlCenter.usbnet.selection.value == "1":
                lines1l2 = "["+results.pop(0)+"]\n"+"netrecoverymode=on\n"
    
            dvbtransmission = str(config.plugins.SundtekControlCenter.dvbtransmission.value)
            if dvbtransmission == "1":
                lines1l3 = "initial_dvb_mode=DVBC\n"
            elif dvbtransmission == "2":
                lines1l3 = "initial_dvb_mode=DVBT\n"
            else:
                lines1l3 = ""  
        
            lines1l4 = ("dreambox_support_fe1=on\n\n")
            stick1 = lines1l1+lines1l2+lines1l3+lines1l4              

        if str(config.plugins.SundtekControlCenter.secondstick.value) == "True":
            if len(results) >= 1:
                twotuners = True

                #stick 2 data
                lines2l1 = ("###### configuration stick 2\n")

                if (not config.plugins.SundtekControlCenter.secusbnet.selection.value == "1"):
                    lines2l2 = ("["+results.pop(0)+"]\n")
                elif config.plugins.SundtekControlCenter.secusbnet.selection.value == "1":
                    lines2l2 = "["+results.pop(0)+"]\n"+"netrecoverymode=on\n"
            
                dvbtransmission = str(config.plugins.SundtekControlCenter.secdvbtransmission.value)
                if dvbtransmission == "1":
                    lines2l3 = "initial_dvb_mode=DVBC\n"
                elif dvbtransmission == "2":
                    lines2l3 = "initial_dvb_mode=DVBT\n"
                else:
                    lines2l3 = ""   
        
                lines2l4 = ("dreambox_support_fe1=on\n")
                stick2 = lines2l1+lines2l2+lines2l3+lines2l4
                
            else: # len < 2, only one or no ip found
                self.session.open(MessageBox, _("Sorry, only one or no stick ID was found.\n\nPlease check that all sticks are plugged correctly. The config file was written without stick information."), MessageBox.TYPE_INFO, 15)
                
                              
        data = header+loglevel+autoupdate+vtuneracceleration+"\n"+netsection+stick1+stick2  
        ### (over)write file
        if (twotuners == True) or (str(config.plugins.SundtekControlCenter.sunconf.support.value) == "True") or (config.plugins.SundtekControlCenter.usbnet.selection.value == "1"):
            f = open(conffile, "w")
            f.writelines(data)
        else:
            if (os.path.exists("/etc/sundtek.conf")):
                self.session.openWithCallback(self.removeconfigfile, MessageBox, _("A configuration file exists.\nYou selected no configuration support.\n\nRemove configuration file /etc/sundtek.conf?"), MessageBox.TYPE_YESNO)
                self.prompt("cat /etc/sundtek.conf")

        
    def removeconfigfile(self, result):
        if result: 
            os.system("rm /etc/sundtek.conf")
            self.session.open(MessageBox, _("Configuration file was deleted."), MessageBox.TYPE_INFO, 7)
                                
                                     

####################################################################


    def setsettings(self):
    
        ### check if driver is installed
                
        if ((not os.path.exists("/opt/bin/mediasrv")) or (not os.path.exists("/opt/bin/mediaclient")) or (not os.path.exists("/usr/sundtek/sun_dvb.sh"))):
            ## maybe driver not installed
            if (os.path.exists("/usr/sundtek/mediasrv") and os.path.exists("/usr/sundtek/mediaclient") and os.path.exists("/usr/sundtek/sun_dvb.sh")) and os.path.exists("/usr/sundtek/libmediaclient.so"):
                ## driver installed but no links in /opt/bin
                if (not os.path.exists("/opt/bin")):
                    os.makedirs("/opt/bin")
                if (not os.path.exists("mkdir /opt/lib")):
                    os.makedirs("/opt/lib")
                os.popen("ln -s /usr/sundtek/mediasrv /opt/bin/mediasrv")
                os.popen("ln -s /usr/sundtek/mediaclient /opt/bin/mediaclient")
                os.popen("ln -s /usr/sundtek/libmediaclient.so /opt/lib/libmediaclient.so ")
            else:
                ## driver not installed
                self.session.openWithCallback(self.installdriverrequest, MessageBox, _("It seems the sundtek driver is not installed or not installed properly. Install the driver now?"), MessageBox.TYPE_YESNO)
            
        else: 
            ### driver installed
            ### disable autostart
            if config.plugins.SundtekControlCenter.autostart.value == False:
                self.prompt("/usr/sundtek/sun_dvb.sh noautostart")
                
            if config.plugins.SundtekControlCenter.usbnet.selection.value == "1":
                ### save the IP for networking
                # f=open("/etc/sundtek.net", "w")
                # networkingip=config.plugins.SundtekControlCenter.usbnet.networkip.value+"\n"
                # networkingip.lstrip().rstrip()
                # f.writelines('REMOTE_IPTV_SERVER='+networkingip)
                # f.close()
                
                if config.plugins.SundtekControlCenter.autostart.value == True:
                    if config.plugins.SundtekControlCenter.dvbtransmission.value == "0": #autostart DVBS
                        self.prompt("/usr/sundtek/sun_dvb.sh autostart_netdvb")
                    else:
                        self.prompt("/usr/sundtek/sun_dvb.sh autostart_net")
                
            else:
                if config.plugins.SundtekControlCenter.dvbtransmission.value == "0":
                    ### dvb-s/ dvb-s2
                    if config.plugins.SundtekControlCenter.autostart.value == True:
                        ### enable autostart
                        self.prompt("/usr/sundtek/sun_dvb.sh autostart_s2")
                        
                elif config.plugins.SundtekControlCenter.dvbtransmission.value == "1":
                    ### dvb-c
                    if config.plugins.SundtekControlCenter.autostart.value == True:
                        ### enable autostart
                        self.prompt("/usr/sundtek/sun_dvb.sh autostart_c")
                else:
                    ### dvb-t
                    if config.plugins.SundtekControlCenter.autostart.value == True:
                        ### enable autostart
                        self.prompt("/usr/sundtek/sun_dvb.sh autostart_t")
                                           
    
    def tunerstart(self):
        for x in self["config"].list:
            x[1].save()
        configfile.save()
        self.sundtekconfigfile()
        self.setsettings()
        
        if (os.path.exists("/opt/bin/mediasrv")) and (os.path.exists("/opt/bin/mediaclient")) and (os.path.exists("/usr/sundtek/sun_dvb.sh")):
            if config.plugins.SundtekControlCenter.dvbtransmission.value == "0":
                ### dvb-s/ dvb-s2
                self.prompt("/usr/sundtek/sun_dvb.sh start_s2")
            elif config.plugins.SundtekControlCenter.dvbtransmission.value == "1":
                ### dvb-c
                self.prompt("/usr/sundtek/sun_dvb.sh start_c")
            else:
                ### dvb-t
                self.prompt("/usr/sundtek/sun_dvb.sh start_t")
            if config.plugins.SundtekControlCenter.usbnet.selection.value == "1":
                ### networking
                if config.plugins.SundtekControlCenter.dvbtransmission.value == "0": #start DVBS
                    self.prompt("/usr/sundtek/sun_dvb.sh start_netdvb")
                else:
                    self.prompt("/usr/sundtek/sun_dvb.sh start_net")
            
    def howtorestart(self):       
        options = [
            (_("Restart E2"), self.restartenigma),
            (_("Restart Dreambox"), self.restartdreambox),           
        ]
        self.session.openWithCallback(self.thismenuCallback, ChoiceBox, list = options)

    def restartenigma(self):
        for x in self["config"].list:
            x[1].save()
        configfile.save()
        self.sundtekconfigfile()
        self.setsettings()
        
        self.session.open(TryQuitMainloop, 3)
        
    def restartdreambox(self):
        for x in self["config"].list:
            x[1].save()
        configfile.save()
        self.sundtekconfigfile()
        self.setsettings()
        
        self.session.open(TryQuitMainloop, 2)            
        
    def dvbinfo(self):
        self.prompt("/usr/sundtek/sun_dvb.sh info")
      
    def installdriverrequest(self, result):
        if result:
            self.session.openWithCallback(self.disclaimer, MessageBox, _("Sundtek legal notice:\nThis software comes without any warranty, use it at your own risk?\nContinue?"), MessageBox.TYPE_YESNO)
            
    def prompt(self, com):
        self.session.open(Console,(""), ["%s" %com])

####################################################################

def main(session, **kwargs):
    session.open(SundtekControlCenter)

def SundtekControlCenterStart(menuid):
    if menuid != "scan": 
        return [ ]
    return [(_("Sundtek Control Center"), main, "Sundtek Control Center", 50)]

def Plugins(path, **kwargs):
    global plugin_path
    plugin_path = path
    list = [
        PluginDescriptor(name=_("sundtek control center plugin"), description =_("installs the sundtek driver and runs related shellscripts"), where = PluginDescriptor.WHERE_MENU, fnc=SundtekControlCenterStart),
        PluginDescriptor(name=_("sundtek control center plugin"), description =_("installs the sundtek driver and runs related shellscripts"), where = PluginDescriptor.WHERE_EXTENSIONSMENU, fnc=main),
        PluginDescriptor(name=_("sundtek control center plugin"), description =_("installs the sundtek driver and runs related shellscripts"), where = PluginDescriptor.WHERE_PLUGINMENU,icon="plugin.png", fnc=main)
        ]
    return list
